<?php

namespace AppDraply\Auth;

use AppDraply\Config;

class OAuth {
	public function register() {
		add_action( 'admin_init', [$this, 'auth'] );
	}

	public function auth() {
		if ( ! isset( $_POST['_wpnonce'] ) || ! is_admin() || ! isset( $_GET['page'] ) || DRAPLY_PLUGIN_SLUG !== $_GET['page'] ) {
			return;
		}

		if ( ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wpnonce'] ) ), Config::$APP_NONCE_SEED ) ) {

			wp_admin_notice( esc_html__( 'The link has expired. Please try again.', 'draply' ), [
				'type' => 'error',
				'dismissible' => true,
			] );

			return;

		}
		wp_redirect( $this->get_auth_url() );

		exit;

	}

	private function get_auth_url(): string {

		$keys = $this->create_keys( Config::$APP_NAME, get_current_user_id(), 'read_write' );

		$path_parameters = [
			'store_name' => sanitize_text_field( get_bloginfo( 'name' ) ),
			'shop_url'   => get_site_url(),
			'api_key'    => $keys['consumer_key'],
			'api_secret' => $keys['consumer_secret'],
			'timezone'   => $this->get_timezone(),
			'currency'   => $this->get_wc_currency(),
			'webhook'    => Config::$WEBHOOK_NAMESPACE . '/' . Config::$WEBHOOK_REST_BASE,
		];

		// $logger = wc_get_logger();
		// $logger->error('Path parameters: ' . json_encode($path_parameters));

		$base64_encoded_json = base64_encode( wp_json_encode( $path_parameters ) );
		return Config::get( 'auth_url' ).'?woo='.$base64_encoded_json;
	}

	private function create_keys( $app_name, $app_user_id, $scope ): array {

		global $wpdb;

		$description = sprintf(
			'%s - API (%s)',
			wc_trim_string( wc_clean( $app_name ), 170 ),
			gmdate( 'Y-m-d H:i:s' )
		);

		$user = wp_get_current_user();

		$permissions     = in_array( $scope, ['read', 'write', 'read_write'], true ) ? sanitize_text_field( $scope ) : 'read';
		$consumer_key    = 'ck_' . wc_rand_hash();
		$consumer_secret = 'cs_' . wc_rand_hash();

		$wpdb->insert(
			$wpdb->prefix . 'woocommerce_api_keys',
			[
				'user_id'         => $user->ID,
				'description'     => $description,
				'permissions'     => $permissions,
				'consumer_key'    => wc_api_hash( $consumer_key ),
				'consumer_secret' => $consumer_secret,
				'truncated_key'   => substr( $consumer_key, -7 ),
			],
			[
				'%d',
				'%s',
				'%s',
				'%s',
				'%s',
				'%s',
			]
		);

		return [
			'key_id'          => $wpdb->insert_id,
			'user_id'         => $app_user_id,
			'consumer_key'    => $consumer_key,
			'consumer_secret' => $consumer_secret,
			'key_permissions' => $permissions,
		];
	}

	private function get_timezone()
	{
			$timezone_str = get_option('timezone_string');

			if (empty($timezone_str)) {
					$gmt_offset = get_option('gmt_offset');
					$timezone_str = timezone_name_from_abbr('', $gmt_offset * 3600, 0);
			}	

			return $timezone_str;
	}


	private function get_wc_currency() {
		if (empty(get_option('woocommerce_currency'))) {
			$logger = wc_get_logger();
			$logger->error('Currency not set in WooCommerce settings');	
			return Config::$DEFAULT_CURRENCY;
		}

		return get_woocommerce_currency();
	}
}
